/***************************************************************************//**
 *   @file   AD5541A.c
 *   @brief  Implementation of AD5541A Driver.
 *   @author Dan Nechita
********************************************************************************
 * Copyright 2012(c) Analog Devices, Inc.
 *
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *  - Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *  - Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *  - Neither the name of Analog Devices, Inc. nor the names of its
 *    contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *  - The use of this software may or may not infringe the patent rights
 *    of one or more patent holders.  This license does not release you
 *    from the requirement that you obtain separate licenses from these
 *    patent holders to use this software.
 *  - Use of the software either in source or binary form, must be run
 *    on or directly connected to an Analog Devices Inc. component.
 *
 * THIS SOFTWARE IS PROVIDED BY ANALOG DEVICES "AS IS" AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, NON-INFRINGEMENT,
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL ANALOG DEVICES BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, INTELLECTUAL PROPERTY RIGHTS, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
********************************************************************************
*   SVN Revision: 786
*******************************************************************************/

/******************************************************************************/
/***************************** Include Files **********************************/
/******************************************************************************/
#include "AD5541A.h"		// AD5541A definitions.

/***************************************************************************//**
 * @brief Initializes the communication peripheral, configures LDAC pin
 *        as output and drives it low.
 *
 * @return Result of the initialization procedure.
 *					Example: 0x0 - SPI peripheral was not initialized.
 *				  			 0x1 - SPI peripheral is initialized.
*******************************************************************************/
unsigned char AD5541A_Init(void)
{
    unsigned char status = 0;
    
    status = SPI_Init(0, 1000000, 1, 0);
    AD5541A_LDAC_OUT;
    AD5541A_LDAC_LOW;
    
    return status;
}

/***************************************************************************//**
 * @brief Sends to DAC register through SPI the 16-bit data-word.
 *
 * @param registerValue - Value of the register.
 *
 * @return none.
*******************************************************************************/
void AD5541A_SetRegisterValue(unsigned short registerValue)
{
    unsigned char registerWord[2] = {0, 0};
    unsigned char* dataPointer = (unsigned char*)&registerValue;

    registerWord[0] = dataPointer[1];
    registerWord[1] = dataPointer[0];
    SPI_Write(AD5541A_SLAVE_ID, registerWord, 2);
}

/***************************************************************************//**
 * @brief Sets the output voltage.
 *
 * @param outVoltage - The voltage value.
 * @param vRef - The voltage reference used by the device.
 *
 * @return Actual voltage that the device can output.
*******************************************************************************/
float AD5541A_SetVoltage(float outVoltage, float vRef)
{
    unsigned short registerValue = 0;
    float          actualVoltage = 0;
    float          code          = 0;
    
    if (vRef == 0)
    {
        vRef = 2.5;
    }
    /* Get raw data from the user's desired voltage value. */
    code = (outVoltage / vRef) * 65536;
    /* Round to the nearest integer. */
    registerValue = (unsigned short)(code + 0.5);
    /* Write to DAC register. */
    AD5541A_SetRegisterValue(registerValue);
    /* Calculate the voltage value that can be outtputed by the device. */
    actualVoltage = vRef * ((float)registerValue / 65536);
    
    return actualVoltage;
}
